#!/usr/bin/python

        # pool_fill_control.py
        ##############################################################
        # Swimming Pool Fill Control Script for a Raspberry Pi 3
        # 
        # V2.0 (2016-05-15)
        # Richard J. Sears
        # richard@sears.net
        ##############################################################
        #
        # This script is part of a smart pool level monitoring system 
        # comprised of a moneino (moteino.com) connected to an eTape
        # water level sensor. The Motenio runs a sketch that monitors
        # the water level in the pool every 1 minutes and if it gets
        # below a certain point, it writes a low water level bit to a 
        # mysql database. 
        #
        # This script reads that bit and then makes a decision what to
        # do based on the water level in the pool.
        #
        # If the water is low, then the script causes GPIO pin 17  
        # to go HIGH powering a relay that opens a sprinkler control
        # valve attached to the pool. It also send a pushbullet 
        # notification so that you know that your pool is being filled.
        #
        # Once the pool level has reached the desired level, the script
        # shuts off the sprinkler valve by forcing the GPIO pin LOW
        # which shuts off the valve. It then sends another pushbullet
        # notification letting you know is has completed filling the
        # pool. 
        #
        # There is basic error checking for running the system too long,
        # and you should ktime how long it takes to fill your pool when 
        # it hits the low level mark and set your maxruntime somewhere
        # above this time, but not TOO far above!
        # 
        # Added blackout period for sprinkler system. Since my pool fills
        # using my irrigation line, I do not want the pool to fill while
        # my sprinklers are running. This would draw too much water
        # pressure from my system.
        #
        ##############################################################



## This is a hobby for me. I am not a programmer and it is quite possible that you could
## flood your yard by using an automatic fill routine. THERE IS NO WARRANTY, use this script
## at your own risk!




# Requires:
# PushBullet Python Library (https://pypi.python.org/pypi/pushbullet.py)
# Free PushBullet Account
# MySQL Python Connector http://http://dev.mysql.com/doc/connector-python/en/connector-python-introduction.html)

import datetime
import mysql.connector
from mysql.connector import errorcode
import time
import RPi.GPIO as GPIO                                         # Import GPIO Library 
import pooldb                                                   # Database information
import alerting                                                 # Pushbullet information
from pushbullet import Pushbullet
import os
import os.path
import threading


## Set up all of our GPIO Stuff here
GPIO.setwarnings(False)                                         # Don't tell me about GPIO warnings. 
GPIO.setmode(GPIO.BCM)                                          # Use BCM Pin Numbering Scheme

## We have a manual fill button with a built-in LED, we set it up here.
MANUAL_FILL_BUTTON = 2                                                  # Our Button is connected to GPIO 2 (Physical Pin 3) Builtin Resistor
MANUAL_FILL_BUTTON_LED = 11                                             # The LED in the button is connected to GPIO 11 (Physical Pin 23)
POOL_FILL_RELAY = 17                                                    #Our relay for the sprinkler valve is on GPIO 17 (Physical Pin 11)

## Setup our GPIO Pins
GPIO.setup(POOL_FILL_RELAY,GPIO.OUT)
GPIO.output(POOL_FILL_RELAY, True)                                      # Set inital state of our relay to off
GPIO.setup(MANUAL_FILL_BUTTON,GPIO.IN)                                  # Make button an input,  since we are using GPIO 2, it has pull up resistor already
GPIO.setup(MANUAL_FILL_BUTTON_LED,GPIO.OUT)                             # Make LED  an Output
GPIO.output(MANUAL_FILL_BUTTON_LED, False)

## Set the Button LED initially off
MANUAL_FILL_BUTTON_LED_ON=False

global current_run_time
current_run_time = 0

# This is where we set up our notifications. I use Pushbullet which is free and very powerful. Visit http://www.pushbullet.com for a free account.
# Once you have your free account, enter your API information in the alerting.py file and restart the script.
pb = Pushbullet(alerting.pushbilletAPI)
def send_notification(fill_status):
                if fill_status =="FILLING" and os.path.isfile("/root/pool_control/alertSent.txt") == False:
                        push = pb.push_note("Swimming Pool is Refilling", "Your swimming pool water level is low and is being refilled.")
                        os.system("touch /root/pool_control/alertSent.txt")
                elif fill_status =="DONE_FILLING" and os.path.isfile("/root/pool_control/alertSent.txt") == True:
                        push = pb.push_note("Swimming Pool Level OK", "Swimming pool water level back to normal. Refilling Complete.")
                        os.system("rm /root/pool_control/alertSent.txt")
                elif fill_status =="MANUAL_FILL":
                        push = pb.push_note("Swimming Pool Manual Fill", "Your swimming pool is being manually filled")
                elif fill_status=="MANUAL_FILL_COMPLETE":
                        push = pb.push_note("Swimming Pool Manual Fill Complete", "Manaul fill of your swimming pool has been completed")
                elif fill_status=="POOL_OVERFILL"and os.path.isfile("/root/pool_control/maxruntime.txt") == False:
                        push = pb.push_note("Swimming Pool Fill Failure", "Your swimming pool might be overfilling! The valve has been DISABLED, you need to reenable it!")


def get_sprinkler_status():
# I use this to keep the pool from filling while my sprinklers are running. You could also use this as a means of
# creating a 'blackout' period during which time you do not want your pool filled. 

# This is the sprinkler start and stop times. Python is a bit weird in that you cannot use 0900 or 0800 because
# of the leading '0'. Any other time is ok.

# Use military time but DO NOT use leading zeros!

        SprinklerStart = int(400)
        SprinklerStop = int(1000)

        current_military_time = int(datetime.datetime.now().strftime('%H%M'))
        print (current_military_time)

        if current_military_time > SprinklerStart and current_military_time < SprinklerStop:
                sprinklers_on = "YES"
        else:
                sprinklers_on = "NO"

        return (sprinklers_on)



# This turns the sprinkler valve on or off when called
def fill_pool(fill_now):
        if fill_now =="START":
                GPIO.output(POOL_FILL_RELAY,False)              #Turns on the sprinkler valve   
        elif fill_now=="STOP":
                GPIO.output(POOL_FILL_RELAY,True)               #Shuts off the sprinkler valve



# Called from the fill_pool() routine and keeps track of how many times we have checked the pool level. 
# We can then decide what to do if it is taking too long to fill the pool (or the sensor node died before
# it could update the database). In this case, it simply flags a file, forces the valve shut and will
# forever do nothing more unless you 1) Manually Fill the pool or 2) remove the maxrruntime.txt file shown
# below and restart the program.
def max_run_time():
        global current_run_time
        current_run_time = current_run_time + 1
        if current_run_time >= pooldb.maxruntime:
                if (alerting.PoolAlerting) == "True":
                        send_notification('POOL_OVERFILL')
                        os.system("touch /root/pool_control/maxruntime.txt")



# This connectes to our database and gets our pool level.
def pool_level():
        if MANUAL_FILL_BUTTON_LED_ON==True:             # Why bother checking if we are manually filling the pool....?
                pass
        else:
                try:
                        cnx = mysql.connector.connect(user=(pooldb.username), password=(pooldb.password), host=(pooldb.servername), database=(pooldb.emoncms_db))
                except mysql.connector.Error as err:
                        if err.errno == errorcode.ER_ACCESS_DENIED_ERROR:
                                print("Something is wrong with your user name or password")
                        elif err.errno == errorcode.ER_BAD_DB_ERROR:
                                print("Database does not exist")
                        else:
                                print(err)
                else:
                        cursor = cnx.cursor(buffered=True)
                        cursor.execute("SELECT data FROM feed_222 ORDER by time DESC LIMIT 1")

                        for (data) in cursor:
                                get_pool_level = ("%1.0f" % (data))
                                cursor.close()

                        if get_pool_level =="1":
                                n = 0
                                print("Pool Level is OK")
                                fill_pool('STOP')
                                if (alerting.PoolAlerting) == "True":
                                        send_notification('DONE_FILLING')
                        elif get_pool_level =="0":
                                print("Pool Level is LOW")
                                sprinkler_status = get_sprinkler_status()
                                if sprinkler_status == "YES":
                                        pass
                                else:
                                        max_run_time()
                                        if os.path.isfile("/root/pool_control/maxruntime.txt") == True:
                                                fill_pool('STOP')
                                        else:
                                                fill_pool('START')              # If the pool level is low, turn on the sprinkler valve
                                                if (alerting.PoolAlerting) == "True":
                                                        send_notification('FILLING')

                        cnx.close()
        threading.Timer(pooldb.checktime, pool_level).start()



# This manages our manaul fill pushbutton
def manual_fill_pool(button):
                global MANUAL_FILL_BUTTON_LED_ON
                if MANUAL_FILL_BUTTON_LED_ON==False:                                      # If the LED is off
                        GPIO.output(MANUAL_FILL_BUTTON_LED,True)                        # turn it on
                        MANUAL_FILL_BUTTON_LED_ON=True                                  # Set Flag to show LED1 is now On 
                        fill_pool('START')
                        if (alerting.PoolAlerting) == "True":
                                 send_notification('MANUAL_FILL')
                else:                                           # If the LED is on
                        GPIO.output(MANUAL_FILL_BUTTON_LED,False)                       # Turn LED off
                        MANUAL_FILL_BUTTON_LED_ON=False                                 # Set Flag to show LED1 is now Off
                        fill_pool('STOP')
                        if (alerting.PoolAlerting) == "True":
                                send_notification('MANUAL_FILL_COMPLETE')


def main():
        pool_level()
        GPIO.add_event_detect(MANUAL_FILL_BUTTON, GPIO.RISING, callback=manual_fill_pool, bouncetime=2000)

main()
                                                                                                                                                                                                   233,1         Bot